--
-- HoseSystemFillArmInteractive
--
-- Authors: Wopster
-- Description: Custom functions for the fillArm
--
-- Copyright (c) Wopster, 2017

HoseSystemFillArmInteractive = {}
HoseSystemFillArmInteractive.MOVEMENT_THRESHOLD = -0.01

---
-- @param specializations
--
function HoseSystemFillArmInteractive.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(HoseSystemVehicle, specializations)
end

function HoseSystemFillArmInteractive:preLoad(savegame)
    self.toggleFillArmLock = HoseSystemFillArmInteractive.toggleFillArmLock
end

function HoseSystemFillArmInteractive:load(savegame)
    local lockAnimation = getXMLString(self.xmlFile, 'vehicle.hoseSystemFillArm#lockAnimationName')

    if lockAnimation ~= nil then
        local movingToolIndice = Utils.getNoNil(getXMLInt(self.xmlFile, 'vehicle.hoseSystemFillArm#movingToolIndice'), 0) + 1

        self.fillArmMovingTool = self.movingTools[movingToolIndice]
        self.fillArmLockAnimation = lockAnimation
    else
        print("HoseSystemFillArmInteractive - invalid lockAnimation!")
    end
end

function HoseSystemFillArmInteractive:postLoad(savegame)
    if savegame ~= nil and not savegame.resetVehicles then
        if self.fillArmLockAnimation ~= nil then
            local lockTime = getXMLInt(savegame.xmlFile, savegame.key .. "#fillArmLockTime")

            if lockTime ~= nil then
                self:setAnimationTime(self.fillArmLockAnimation, lockTime, true)
            end
        end
    end
end

function HoseSystemFillArmInteractive:delete()
end

function HoseSystemFillArmInteractive:getSaveAttributesAndNodes(nodeIdent)
    local attributes

    if self.fillArmLockAnimation ~= nil then
        attributes = ('fillArmLockTime="%s"'):format(self:getAnimationTime(self.fillArmLockAnimation))
    end

    return attributes, nil
end

function HoseSystemFillArmInteractive:mouseEvent(...)
end

function HoseSystemFillArmInteractive:keyEvent(...)
end

function HoseSystemFillArmInteractive:update(dt)
    if self.fillArmLockAnimation == nil then
        return
    end

    if self:getIsActive() and self:getIsActiveForInput() and not self:hasInputConflictWithSelection() then
        if self.fillArmMovingTool ~= nil and self.fillArmMovingTool.curRot[1] > HoseSystemFillArmInteractive.MOVEMENT_THRESHOLD then
            if InputBinding.hasEvent(InputBinding.unlockArm) then
                self:toggleFillArmLock()
            end
        end
    end
end

function HoseSystemFillArmInteractive:updateTick(dt)
    if self.fillArmLockAnimation == nil then
        return
    end

    if self.fillArmMovingTool ~= nil then
        if self:getIsAnimationPlaying(self.fillArmLockAnimation) then
            self:setMovingToolDirty(self.fillArmMovingTool.node)
        end

        if self.fillArmMovingTool.curRot[1] > HoseSystemFillArmInteractive.MOVEMENT_THRESHOLD then
            if self:getAnimationTime(self.fillArmLockAnimation) > 0.5 then
                if not self.fillArmMovingTool.isActive then
                    self.fillArmMovingTool.isActive = true
                end
            else
                if self.fillArmMovingTool.isActive then
                    self.fillArmMovingTool.isActive = false
                end
            end
        end
    end
end

function HoseSystemFillArmInteractive:toggleFillArmLock()
    if not self.getIsAnimationPlaying(self, self.fillArmLockAnimation) then
        local dir = self:getAnimationTime(self.fillArmLockAnimation) == 1 and -1 or 1

        self:playAnimation(self.fillArmLockAnimation, dir)
    end
end

function HoseSystemFillArmInteractive:draw()
    if self.fillArmLockAnimation ~= nil and self.fillArmMovingTool.curRot[1] > HoseSystemFillArmInteractive.MOVEMENT_THRESHOLD then
        local text

        if self:getAnimationTime(self.fillArmLockAnimation) == 0 and InputBinding.getIsInputMouseAxisTriggered(InputBinding[self.fillArmMovingTool.axis]) then
            text = g_i18n:getText("action_unlockArm")
        elseif self:getAnimationTime(self.fillArmLockAnimation) > 0 then
            text = g_i18n:getText("action_lockArm")
        end

        if text ~= nil then
            g_currentMission:showBlinkingWarning(text, 0)
            g_currentMission:addHelpButtonText(text, InputBinding.unlockArm)
        end
    end
end