--
-- PressureCylinder
--
-- Authors: Wopster
-- Description: physical pressure cylinder connection
--
-- Copyright (c) Wopster, 2017

PressureCylinder = {}

PressureCylinder.DISTANCE = 1.5

function PressureCylinder.prerequisitesPresent(specializations)
    if not SpecializationUtil.hasSpecialization(Cylindered, specializations) then
        print("Warning: Specialization PressureCylinder needs the specialization Cylindered.")

        return false
    end

    return true
end

function PressureCylinder:load(savegame)
    if self.isClient then
        local rotationPartId = getXMLInt(self.xmlFile, "vehicle.pressureCylinder#rotationPartId")
        local referenceIds = Utils.getVectorNFromString(getXMLString(self.xmlFile, "vehicle.pressureCylinder#referenceIds"))

        if rotationPartId ~= nil and referenceIds ~= nil then
            table.insert(referenceIds, rotationPartId)

            self.pressureCylinder = {
                rotationPartId = rotationPartId,
                referenceIds = referenceIds,
                dummyReferenceNode = nil,
                closestAttacherJointId = nil
            }
        else
            print("PressureCylinder error: invalid rotationPartId or referenceIds!")
        end
    end
end

function PressureCylinder:delete()
end

function PressureCylinder:mouseEvent(...)
end

function PressureCylinder:keyEvent(...)
end

function PressureCylinder:update(dt)
end

function PressureCylinder:draw()
end

---
-- @param attacherJoints
-- @param partTrans
--
function PressureCylinder.getClosestTopArmAttacherJoint(attacherJoints, partTrans)
    local closestId
    local closestDistance = PressureCylinder.DISTANCE

    for id, jointDesc in ipairs(attacherJoints) do
        if jointDesc.topArm ~= nil then
            local trans = { getWorldTranslation(jointDesc.topArm.rotationNode) }
            local distance = Utils.vector3Length(trans[1] - partTrans[1], trans[2] - partTrans[2], trans[3] - partTrans[3])

            if distance < closestDistance then
                closestId = id
                closestDistance = distance
            end
        end
    end

    return closestId
end

---
-- @param attacherVehicle
-- @param jointDescIndex
--
function PressureCylinder:onAttach(attacherVehicle, jointDescIndex)
    if self.isClient and self.pressureCylinder ~= nil then
        local closestId
        local movingPart = self.movingParts[self.pressureCylinder.rotationPartId].referencePoint
        local partTrans = { getWorldTranslation(movingPart) }

        if self.pressureCylinder.dummyReferenceNode == nil then
            self.pressureCylinder.dummyReferenceNode = movingPart
        end

        if attacherVehicle.attacherJoints[jointDescIndex].topArm == nil then
            closestId = PressureCylinder.getClosestTopArmAttacherJoint(attacherVehicle.attacherJoints, partTrans)
        else
            closestId = jointDescIndex
        end

        if closestId ~= nil then
            local attacherJoint = attacherVehicle.attacherJoints[closestId]

            if attacherJoint ~= nil and attacherJoint.topArm ~= nil then
                for _, id in ipairs(self.pressureCylinder.referenceIds) do
                    self.movingParts[id].referencePoint = attacherJoint.topArm.rotationNode
                end

                self.pressureCylinder.closestAttacherJointId = closestId

                setVisibility(attacherJoint.topArm.rotationNode, false)
            end
        end
    end
end

---
-- @param attacherVehicle
--
function PressureCylinder:onDetach(attacherVehicle)
    if self.isClient and self.pressureCylinder ~= nil and self.pressureCylinder.closestAttacherJointId ~= nil then
        local attacherJoint = attacherVehicle.attacherJoints[self.pressureCylinder.closestAttacherJointId]

        for _, id in ipairs(self.pressureCylinder.referenceIds) do
            self.movingParts[id].referencePoint = self.pressureCylinder.dummyReferenceNode
        end

        setVisibility(attacherJoint.topArm.rotationNode, true)

        self.pressureCylinder.closestAttacherJointId = nil

        self:updateCylinderedInitial()
    end
end
